/**
        \file cli.c
        \author Sachith Dunatunga
        \date 21.04.09

        The command line version of p2kc-linux, based on libparagon.
*/
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <readline/readline.h>
#include <readline/history.h>
#include "arguments.h"
#include "commands.h"
#include "paragon.h"

#define COMMENT_MARKER '#'

/* ------------------------------------------------------------------------ */
void usage(char *name);
void logger(int level, const char *msg, va_list ap);
int clear_cmd(int argc, char **argv);
int exit_cmd(int argc, char **argv);
int help_cmd(int argc, char **argv);
int upload_cmd(int argc, char **argv);

const char default_modem_device[] = "/dev/ttyACM0";
const char history_file[] = ".p2kc-linux.history";
FILE *logfp = NULL;
paragon_handle_t *lib = NULL;
int loglevel = PL_ERROR;

const command_t commands[] = {
        {"clear", clear_cmd, 0, NULL},
        {"exit", exit_cmd, 0, NULL},
        {"help", help_cmd, 0, NULL},
        {"upload", upload_cmd, 0, NULL}
};

int clear_cmd(int argc, char **argv)
{
        if (argc > 1) {
                if (0 == strcmp(argv[1], "history")) {
                        clear_history();
                        return 0;
                }
        }
        printf("Type '%s history' (without quotes) to delete"
                " the history.\n", argv[0]);
        return 0;
}

int exit_cmd(int argc, char **argv)
{
        if (argc != 1) {
                printf("Type \'exit\' (without quotes) to exit.\n");
        }
        return 0;
}

int help_cmd(int argc, char **argv)
{
        int i;
        int number_of_commands = sizeof(commands) / sizeof(commands[0]);

        printf("Valid commands:\n");
        for (i = 0; i < number_of_commands; i++) {
                printf("%s ", commands[i].name);
        }
        printf("\n");
        printf("For help on a specific command, type \'<command> help\'\n");
        return 0;
}

int upload_cmd(int argc, char **argv)
{
        int i, n;
        FILE *f;
        unsigned char buffer[1024];
        struct stat file_stats;
        int file_size;

        if (argc != 3) {
                printf("%s: mobile-file local-file\n", argv[0]);
                return 0;
        }

        /*
                Set the correct file mask.
        */
        paragon_set_file_mask(lib, argv[1][1], "*", &n);

        /*
                Default attributes.
        */
        paragon_open_file(lib, argv[1], 0);

        if (0 != stat(argv[2], &file_stats)) {
                printf("Can't open local-file. Aborting upload.\n");
                return 0;
        }

        file_size = file_stats.st_size;

        f = fopen(argv[2], "rb");
        i = 0;
        while ((n = fread(buffer, 1, 1024, f))) {
                i += n;
                paragon_write_file(lib, buffer, n);
                printf("\r[%3d%%]\t%10d/%d bytes written to phone.",
                        100 * i / file_size, i, file_size);
                fflush(stdout);
        }
        printf("\n");

        if (i != file_size) {
                printf("Error occured; partial file on phone.\n");
        }

        fclose(f);
        paragon_close_file(lib);

        return 0;
}

void usage(char *name)
{
        fprintf(stderr,
        "%s: [-l [LOGFILE]] [-m MODE] [-p PATH] [-v LEVEL]\n",
                name);

        fprintf(stderr, "A program to access Motorola phones.\n");

        fprintf(stderr,
        "  -h           This help message.\n");

        fprintf(stderr,
        "  -l           Write the libparagon log to LOGFILE. If LOGFILE is\n"
        "               \"stderr\", write to standard error\n");

        fprintf(stderr,
        "  -m           Assume the phone is in mode MODE prior to\n"
        "               initialization. Possible values for mode:\n"
        "                       auto    Scan all modes (default)\n"
        "                       at      Modem mode, no kernel drivers\n"
        "                       p2k     P2K or P2K05 mode\n"
        "                       modem   Modem mode, cdc_acm kernel driver\n");

        fprintf(stderr,
        "  -p           The path to the modem device; the default is\n"
        "               \"%s\".\n", default_modem_device);

        fprintf(stderr,
        "  -v           If -l is enabled, controls which messages appear in\n"
        "               log output. It is formed by masking the following\n"
        "               values:\n"
        "                       0x%02x    Errors (default)\n"
        "                       0x%02x    Warnings\n"
        "                       0x%02x    Information\n",
        PL_ERROR, PL_WARNING, PL_INFO);

        exit(EXIT_FAILURE);
        return;
}

void logger(int level, const char *msg, va_list ap)
{
        struct timeval tv;
        int secs, usecs;

        if (logfp == NULL) {
                return;
        }

        /*
                Check if it's one of the levels we care about.
        */
        if (!(level & loglevel)) {
                return;
        }

        /*
                Setup a little timestamp for logging purposes.
        */
        gettimeofday(&tv, NULL);
        secs = tv.tv_sec % 100000;
        usecs = (int)tv.tv_usec;
        fprintf(logfp, "[%05d.%06d]: ", secs, usecs);

        /*
                I like the way XFree86 logs messages; I try to partially
                duplicate it here.
        */
        switch (level) {
                case PL_ERROR:
                        fprintf(logfp, "(EE) ");
                break;
                case PL_WARNING:
                        fprintf(logfp, "(WW) ");
                break;
                case PL_INFO:
                        fprintf(logfp, "(II) ");
                break;
                default:
                        fprintf(logfp, "(%d) ", level);
        }

        vfprintf(logfp, msg, ap);
        fflush(logfp);

        return;
}

int main(int argc, char **argv)
{
        paragon_handle_t hlib;
        paragon_init_t p;
        args_t *arg = NULL;
        char *logfile = NULL;
        char *line = NULL;
        char *home_directory;
        char *abs_path_history_file;
        int exit_flag = 0;
        int number_of_commands = sizeof(commands) / sizeof(commands[0]);
        int exit_result;
        int result;
        int i, c;

        /*
                Set our global handle up.
        */
        lib = &hlib;

        /*
                Setup our parameters.
        */
        p.acm_dev_path = (char *)default_modem_device;
        p.initial_mode = PARAGON_MODE_AUTO;
        p.vendor_id = PARAGON_ANY_VENDOR;
        p.product_id = PARAGON_ANY_PRODUCT;
        p.logger = &logger;

        home_directory = getenv("HOME");

        /*
                One extra character for terminating null, one extra for
                slash.
        */
        abs_path_history_file =
                malloc(strlen(history_file) + strlen(home_directory) + 2);

        if (NULL == abs_path_history_file) {
                fprintf(stderr, "Out of memory. Exiting.\n");
                return EXIT_FAILURE;
        }

        strcpy(abs_path_history_file, home_directory);
        strcat(abs_path_history_file, "/");
        strcat(abs_path_history_file, history_file);

        /*
                We use getopt here to do the hard work of parsing arguments
                for us.
        */
        while (-1 != (c = getopt(argc, argv, "hl:m:p:v:"))) {
                switch (c) {
                        case 'l':
                                if (0 == strcmp(optarg, "stderr")) {
                                        logfp = stderr;
                                } else {
                                        logfile = optarg;
                                        logfp = fopen(logfile, "a");
                                }
                        break;
                        case 'm':
                                if (0 == strcmp(optarg, "auto")) {
                                        p.initial_mode = PARAGON_MODE_AUTO;
                                } else if (0 == strcmp(optarg, "at")) {
                                        p.initial_mode = PARAGON_MODE_AT;
                                } else if (0 == strcmp(optarg, "p2k")) {
                                        p.initial_mode = PARAGON_MODE_P2K;
                                } else if (0 == strcmp(optarg, "modem")) {
                                        p.initial_mode = PARAGON_MODE_ACM;
                                }
                        break;
                        case 'p':
                                p.acm_dev_path = optarg;
                        break;
                        case 'v':
                                /*
                                        Strtol() is slightly smarter than
                                        atoi, and will change bases if the
                                        number is formatted accordingly. ie
                                        0x12    -> 12 HEX -> 18 DEC
                                        12      -> 12 DEC -> 12 DEC
                                        012     -> 12 OCT -> 10 DEC
                                */
                                loglevel = strtol(optarg, NULL, 0);
                        break;
                        case 'h':
                        case '?':
                        default:
                                usage(argv[0]);
                }
        }

        /*
                Setup libparagon with our parameters.
        */
        result = paragon_init(&hlib, &p);
        if (PARAGON_SUCCESS != result) {
                printf("libparagon error -- %s\n", paragon_strerror(result));
                exit_result =  EXIT_FAILURE;
                goto init_fail;
        }

        /*
                Try to set the phone to P2K/P2K05 mode so we can do
                interesting things with it.
        */
        printf("Attempting mode switch (wait 5-7 seconds)...\n");
        result = paragon_set_mode(&hlib, PARAGON_MODE_P2K);
        if (PARAGON_SUCCESS != result) {
                printf("libparagon error -- %s\n", paragon_strerror(result));
                exit_result = EXIT_FAILURE;
                goto p2k_mode_fail;
        }

        /*
                Alright, the phone is in p2k mode.
        */
        printf("Success. Phone is in P2K Mode.\n");

        /*
                Read the command history if it exists.
        */
        read_history(abs_path_history_file);

        do {
                line = NULL;
                arg = NULL;

                line = readline("P2K:$ ");
                arg = args_parse_simple(line);

                if (NULL == arg) {
                        /*
                                Something's wrong, try it again.
                        */
                        free(line);
                        continue;
                }

                if (0 == arg->argc) {
                        args_cleanup(arg);
                        free(line);
                        continue;
                }

                /*
                        We will try to support 'scripting' commands by simply
                        reading from standard input via < or cat; treat any
                        line that starts with '#' as a comment and ignore it.
                */
                if (COMMENT_MARKER == arg->argv[0][0]) {
                        args_cleanup(arg);
                        free(line);
                        continue;
                }

                for (i = 0; i < number_of_commands; i++) {
                        if (0 == strcmp(commands[i].name, arg->argv[0])) {
                                break;
                        }
                }

                /*
                        We found a match in the commands structure. Execute
                        the command with the arguments.
                */
                if (i < number_of_commands) {
                        if (NULL != commands[i].func) {
                                (*commands[i].func)(arg->argc, arg->argv);
                        }
                } else {
                        printf("%s: command not found\n", arg->argv[0]);
                }

                /*
                        The exit function is the only one we'll special case.
                        Everything else gets handled automatically by the
                        for loop. We do this because the exit function is the
                        only one that affects the running state of the
                        program.
                */
                if (0 == strcmp(line, "exit")) {
                        exit_flag = 1;
                }

                /*
                        Add the command to the history file.
                */
                add_history(line);

                /*
                        Free our allocated memory.
                */
                args_cleanup(arg);
                free(line);
        } while (!exit_flag);

        /*
                Write the command history to our file.
        */
        write_history(abs_path_history_file);

        /*
                Let's return the phone to AT mode before we exit (to be a
                well-behaving application).
        */
        printf("Returning to AT mode (wait 5-7 seconds)...\n");
        result = paragon_set_mode(&hlib, PARAGON_MODE_AT);
        if (PARAGON_SUCCESS != result) {
                printf("libparagon error -- %s\n", paragon_strerror(result));
                exit_result = EXIT_FAILURE;
                goto at_mode_fail;
        }

        exit_result = EXIT_SUCCESS;

        at_mode_fail:
        p2k_mode_fail:

        /*
                We're done with libparagon, so deinitialize it.
        */
        paragon_fini(&hlib);

        init_fail:

        /*
                Free the memory we grabbed for our history file path.
        */
        free(abs_path_history_file);

        /*
                Close our log file handle if we were using one.
        */
        if (NULL != logfp && stderr != logfp) {
                fflush(logfp);
                fclose(logfp);
        }

        return exit_result;
}
/* ------------------------------------------------------------------------ */
