/**
        \file p2k_example.c
        \author Sachith Dunatunga
        \date 21.04.09

        An example file to show how to use libparagon.
*/
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <sys/time.h>
#include <unistd.h>
#include "paragon.h"

#define PHONE_ID_SEEM_NUMBER 0x117
#define PHONE_ID_SEEM_RECORD 0x01

/* ------------------------------------------------------------------------ */
FILE *fp = NULL;

void usage(char *name);
void file_logger(int level, const char *msg, va_list ap);
void print_phone_id(unsigned char *seemdata);

void print_phone_id(unsigned char *seemdata)
{
        int i = 1;
        char c;

        if (NULL == seemdata) {
                return;
        }

        printf("Phone ID is: ");
        /*
                The phone id format seems to be ucs-2; the quick way to
                format it into ascii is grab every other byte starting from
                the second byte, but this is really an egregious hack.
        */
        do {
                c = seemdata[i];
                printf("%c", c);
                i += 2;
        } while (0 != c);

        printf("\n");

        return;
}

void usage(char *name)
{
        fprintf(stderr, "%s: [-h] [-l LOGFILE] [-n NUM] [-w FILE]\n", name);
        fprintf(stderr, "An example program using libparagon.\n");
        fprintf(stderr, " -l\tWrite the libparagon log to LOGFILE.\n");
        fprintf(stderr, " -n\tRead the seem NUM times.\n");
        fprintf(stderr, " -w\tWrite the seem data to FILE\n");
        fprintf(stderr, " -h\tThis help message.\n");
        exit(EXIT_FAILURE);
        return;
}

void file_logger(int level, const char *msg, va_list ap)
{
        struct timeval tv;
        int secs, usecs;

        if (fp == NULL) {
                return;
        }

        /*
                Setup a little timestamp for logging purposes.
        */
        gettimeofday(&tv, NULL);
        secs = tv.tv_sec % 100000;
        usecs = (int)tv.tv_usec;
        fprintf(fp, "[%05d.%06d]: ", secs, usecs);

        switch (level) {
                case PL_ERROR:
                        fprintf(fp, "(EE) ");
                        fprintf(stderr, "(EE) ");
                        vfprintf(stderr, msg, ap);
                break;
                case PL_WARNING:
                        fprintf(fp, "(WW) ");
                        fprintf(stderr, "(WW) ");
                        vfprintf(stderr, msg, ap);
                break;
                case PL_INFO:
                        fprintf(fp, "(II) ");
                break;
                default:
                        fprintf(fp, "(%d) ", level);
        }

        vfprintf(fp, msg, ap);
        fflush(fp);

        return;
}

int main(int argc, char **argv)
{
        paragon_handle_t libhandle;
        paragon_init_t params;
        paragon_seem_t phone_id;
        FILE *seemfp;
        char *logfile = NULL;
        char *seemfile = NULL;
        const char *opts = "hl:n:w:";
        int mode = 0;
        int did_write = 0;
        int result;
        int n = 1;
        int i;
        int c;

        int files;
        paragon_file_record_t fr;

        unsigned char *file_data;
        int file_size;
        int fsize_max = 0;
        FILE *dumpfile;

        /*
                Setup our parameters.
        */
        params.acm_dev_path = NULL;
        params.initial_mode = PARAGON_MODE_AUTO;
        params.vendor_id = PARAGON_ANY_VENDOR;
        params.product_id = PARAGON_ANY_PRODUCT;
        params.logger = &file_logger;

        /*
                We use getopt here to do the hard work of parsing arguments
                for us.
        */
        while (-1 != (c = getopt(argc, argv, opts))) {
                switch (c) {
                        case 'l':
                                logfile = optarg;
                        break;
                        case 'n':
                                n = atoi(optarg);
                                if (n < 1) {
                                        n = 1;
                                }
                        break;
                        case 'w':
                                seemfile = optarg;
                        break;
                        case 'h':
                        case '?':
                        default:
                                usage(argv[0]);
                }
        }

        if (NULL != logfile) {
                fp = fopen(logfile, "a");
        }

        /*
                Setup libparagon.
        */
        result = paragon_init(&libhandle, &params);
        if (PARAGON_SUCCESS != result) {
                printf("libparagon error -- %s\n", paragon_strerror(result));
                return EXIT_FAILURE;
        }

        result = paragon_get_mode(&libhandle, &mode);
        if (PARAGON_SUCCESS != result) {
                printf("libparagon error -- %s\n", paragon_strerror(result));
                return EXIT_FAILURE;
        }

        /*
                Check which mode the phone is in right now.
        */
        switch (mode) {
                case PARAGON_MODE_AT:
                        printf("Phone is in AT mode.\n");
                break;
                case PARAGON_MODE_ACM:
                        printf("Phone is in ACM mode.\n");
                break;
                case PARAGON_MODE_MEMCARD:
                        printf("Phone is in Memcard mode.\n");
                break;
                case PARAGON_MODE_P2K:
                        printf("Phone is in P2K mode.\n");
                break;
                default:
                        printf("Phone is in unknown mode.\n");
        }

        /*
                Try to set the phone to the correct mode.
        */
        printf("Attempting mode switch (wait 5-7 seconds)...\n");
        result = paragon_set_mode(&libhandle, PARAGON_MODE_P2K);

        if (PARAGON_SUCCESS == result) {
                printf("Success. Phone is in P2K Mode.\n");

                printf("Volume /a JPG File listing (Skipping Directories):\n");
                paragon_set_file_mask(&libhandle, 'e', "*.jpg", &files);
                printf("Attributes\tOwner\tSize\tPath\n");
                paragon_close_file(&libhandle);
                for (i = 0; i < files; i++) {
                        paragon_get_next_file_record(&libhandle, &fr);
                        if (!PARAGON_IS_DIR(fr.attributes)) {
                                paragon_open_file(&libhandle, fr.path, fr.attributes);
                                printf("%x\t\t%x\t%d\t%s\n", fr.attributes, fr.owner, fr.size, fr.path);
                                file_size = 0;
                                paragon_read_file(&libhandle, fr.size, &file_data, &file_size);
                                if (file_size > fsize_max) {
                                        dumpfile = fopen("dumpfile.out", "wb+");
                                        fsize_max = file_size;
                                        fwrite(file_data, file_size, 1, dumpfile);
                                        fflush(dumpfile);
                                        fclose(dumpfile);
                                }
                                paragon_close_file(&libhandle);
                                free(file_data);
                        }
                }

                for (i = 0; i < n; i++) {
                        phone_id.data = NULL;
                        printf("Reading PhoneID Seem (0x%04x, 0x%04x)"
                                " [attempt %d]: \n", PHONE_ID_SEEM_NUMBER,
                                PHONE_ID_SEEM_RECORD, i + 1);

                        /*
                                The phone is in the right mode, now try to
                                read the Phone ID seem.
                        */
                        result = paragon_read_seem(&libhandle,
                                PHONE_ID_SEEM_NUMBER, PHONE_ID_SEEM_RECORD,
                                &phone_id);

                        if (PARAGON_SUCCESS == result) {
                                printf("Success.\n");
                                printf("Seem Number: %d\n", phone_id.number);
                                printf("Seem Record: %d\n", phone_id.record);
                                printf("Seem Offset: %d\n", phone_id.offset);
                                printf("Seem Size: %d\n", phone_id.data_len);
                                if (seemfile != NULL && !did_write) {
                                        seemfp = fopen(seemfile, "wb");
                                        fwrite(phone_id.data,
                                                phone_id.data_len,
                                                sizeof(char), seemfp);
                                        did_write = 1;
                                }
                                print_phone_id(phone_id.data);
                        } else {
                                printf("Failure.\n");
                        }

                        paragon_log(&libhandle, PL_INFO,
                                "Seem Data [attempt %d] is:\n", i);
                        paragon_print_hexdump(&libhandle, phone_id.data,
                                phone_id.data_len);
                        free(phone_id.data);
                }

                /*
                        Try to return the phone to AT mode.
                */
                printf("Switching to AT mode (wait 5-7 seconds)...\n");
                result = paragon_set_mode(&libhandle, PARAGON_MODE_AT);
                if (PARAGON_SUCCESS == result) {
                        printf("Success. Phone is in AT Mode.\n");
                } else {
                        printf("Failure -- %s\n", paragon_strerror(result));
                }
        } else {
                printf("Failure -- %s\n", paragon_strerror(result));
        }

        /*
                We're done with libparagon, so deinitialize it.
        */
        paragon_fini(&libhandle);
        return EXIT_SUCCESS;
}
/* ------------------------------------------------------------------------ */
