/**
        \file logger.h
        \author Sachith Dunatunga
        \date 21.04.09
        \brief The header/interface for the libparagon logger.
*/
#ifndef __PARAGON_LOGGER_H__
#define __PARAGON_LOGGER_H__
#include <stdarg.h>
#include "handle.h"

/* ---libparagon----------------------------------------------------------- */
/**
        \defgroup loglevels Logging Levels
                The different levels of logging error messages.
        @{
*/
/**
        \brief The message that follows details an error. Typically this is
        used if the function fails after the logging call.
*/
#define PL_ERROR (0x01)
/**
        \brief The message that follows details a warning. This is used if
        the function can continue despite some condition.
*/
#define PL_WARNING (0x02)
/**
        \brief The message that follows details some extra information. This
        is used if the function can continue as normal, but some non-critical
        extra data may be communicated to the user.
*/
#define PL_INFO (0x04)
/**
        @}
*/

/**
        \brief This function merely acts as a wrapper for the custom logging
        function (if any).
        \param handle The libparagon handle to use.
        \param level The severity of the message.
        \param msg The message to send. It takes a format string like printf.

        This function just acts as a wrapper to handle->logger if it is not
        NULL. Otherwise it does nothing.
*/
void paragon_log(paragon_handle_t *handle, int level, const char *msg, ...);
/**
        \brief The default logger for libparagon.
        \param level The loglevel.
        \param msg A printf-style format string.
        \param ap The varargs list (for vprintf).

        This is the default logging function for libparagon. It just prints
        out all messages to stderr.
*/
void paragon_default_logger(int level, const char *msg, va_list ap);
/**
        \brief Prints out a buffer in hex and ascii representations.
        \param handle The libparagon handle to use.
        \param data The buffer to print.
        \param bytes The size of 'data' in bytes.
        \warning This should only be used in console programs, as it relies
        on sprintf().

        Pretty-prints a hex representation of the buffer.
*/
void paragon_print_hexdump(paragon_handle_t *handle, unsigned char *data,
        int bytes);
/* ------------------------------------------------------------------------ */
#endif
