/**
        \file modem.c
        \author Sachith Dunatunga
        \date 17.04.09
        \brief The implementation of the modem interaction functions.
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <termios.h>
#include <unistd.h>
#include <fcntl.h>
#include <ctype.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "modem.h"
#include "timings.h"

/* ---modem---------------------------------------------------------------- */
/**
        \brief This parameter is used during initialization to determine
        how many attempts should be made to read the modem after the
        initialization string is sent.
*/
const int MODEM_READ_RETRIES = 5;

/**
        \brief Sends the initialization commands to the modem and sets
        the communication parameters.
        \param handle The modem to initialize.
        \return -1 on error, 0 on success.

        This function sends an AT string with specific parameters and
        additionally sets the terminal settings to match the initialzation
        string. It also will attempt to store the current settings so we
        can restore them when we are done with the modem.
*/
int modem_init(modem_t *handle);
/**
        \brief Returns the modem to pre-initialization state (as much as
        possible).
        \param handle The modem to return to pre-init state.
        \return -1 on error, 0 on success.

        This function takes the old settings we saved during initialization
        and reapplies them.
*/
int modem_cleanup(modem_t *handle);
/**
        \brief Scans a string and returns the last line of input.
        \param str The string to scan.
        \return A pointer inside of 'str' that corresponds to the last line.

        If the input string is newline terminated, it ignores that newline
        when searching for the last newline. This function does not alter the
        string passed in, but care should be taken passing the output of this
        function to functions that may alter the string.
*/
char *getlastline(char *str);
/**
        \brief Scans a string and replaces all intances of one character with
        another.
        \param str The string to scan.
        \param to The character replacing 'from'.
        \param from The character to look for and replace.
        \param len The maximum number of characters to scan in 'str'.
        \return The string with 'to' replacing 'from'.
        \warning This function is for internal implementation only.

        The strnrepl function alters 'str', so if you need the original string
        you should send it a copy. As an example of usage, calling \code
        char str[] = "cat";
        strnrepl(str, 'r', 't', 3); \endcode results in str containing "car".
*/
char *strnrepl(char *str, char to, char from, int len);
/**
        \brief Trims the string of trailing and leading whitespace.
        \param str The string to trim.
        \return A new string with no leading or trailing whitespace.
        \warning This function is for internal implementation only.

        The trim function does not alter the original string; rather it
        returns a copy which must you must free() to avoid memory leaks.
*/
char *trim(char *str);

char *getlastline(char *str)
{
        int len;
        int c;

        if (NULL == str) {
                return NULL;
        }

        len = strlen(str);
        c = len;

        /*
                Start from the end of the string and work backwards until we
                find a newline character.
        */
        while (c > 0) {
                if (str[--c] == '\n') {
                        /*
                                If the newline immediately preceeds the null
                                terminator, ignore it; it's a terminating
                                newline. Otherwise we have found our last
                                line.
                        */
                        if (c + 1 != len) {
                                c++;
                                break;
                        }
                }
        }

        return (str + c);
}

char *strnrepl(char *str, char to, char from, int len)
{
        int i = 0;

        /* Save ourselves some work if we replace a character with itself. */
        if (NULL == str || to == from) {
                return str;
        }

        /*
                We scan the string looking for the 'from' characters and
                replace them with the 'to' characters.
        */
        while (i < len) {
                if (str[i] == from) {
                        str[i] = to;
                }
                i++;
        }

        return str;
}

char *trim(char *str)
{
        int b, e;
        char *ret;

        if (NULL == str) {
                return NULL;
        }

        /* The beginning and ending of the string (used for pointers). */
        b = 0;
        e = strlen(str) - 1;

        /*
                Scan the beginning of the string for whitespace until we
                encounter something that isn't whitespace.
        */
        while (isspace(str[b])) {
                b++;
                /*
                        This is a problem; if the beginning and ending
                        pointers cross, the string only consists of
                        whitespace, so return NULL.
                */
                if (b > e) {
                        return NULL;
                }
        }

        /*
                Scan the end of the string for whitespace going backwards.
                We aren't worried that 'e' goes less than zero because the
                case that 'b' and 'e' cross is taken care of above and b
                starts at 0.
        */
        while (isspace(str[e])) {
                e--;
        }
        e++;

        /*
                Allocate memory for the new string and fill it in with the
                relevant characters. Return it to the caller.
        */
        ret = (char *)malloc(e - b + 1);
        memcpy(ret, str + b, e - b);
        ret[e - b] = '\0';
        return ret;
}

modem_t *modem_open(char *modem)
{
        modem_t *handle;

        handle = malloc(sizeof(modem_t));

        if (NULL == handle) {
                return NULL;
        }

        /* Open the modem device just as a normal file. */
        handle->fd = open(modem, O_RDWR | O_NOCTTY | O_NDELAY);

        if (-1 != handle->fd) {
                /*
                        We issue this command to clear the file descriptor
                        flags.
                */
                fcntl(handle->fd, F_SETFL, 0);
                if (0 != modem_init(handle)) {
                        free(handle);
                        handle = NULL;
                }
        } else {
                free(handle);
                handle = NULL;
        }

        return handle;
}

int modem_close(modem_t *handle)
{
        int result = 0;

        if (handle == NULL) {
                return result;
        }

        /* Do not attempt to close a descriptor which is clearly in error. */
        if (-1 != handle->fd) {
                result = modem_cleanup(handle);
                if (-1 != result) {
                        result = close(handle->fd);
                }
        }

        return result;
}

int modem_init(modem_t *handle)
{
        char *response;
        char *parsed;
        int init_ok;
        int retries;
        int len;
        /*
                This initialization string is meant to correspond to the
                settings we set throughout this function; however in practice
                the modem may not support all of these commands.
        */
        char init_string[] = "AT S7=45 S0=0 L1 V1 X4 &c1 E0 Q0\r";
        /*
                This is what we expect the modem to respond with.
        */
        char expected[] = "OK";

        if (NULL == handle) {
                return -1;
        }

        if (-1 == handle->fd) {
                return -1;
        }

        /*
                Save the old settings for use by modem_cleanup().
        */
        if (-1 == tcgetattr(handle->fd, &(handle->old_settings))) {
                return -1;
        }

        /*
                Duplicate our settings for alteration.
        */
        if (-1 == tcgetattr(handle->fd, &(handle->settings))) {
                return -1;
        }

        /*
                Set the input and output speed of the connection to 115200
                baud.
        */
        if (-1 == cfsetispeed(&(handle->settings), B115200)) {
                return -1;
        }
        if (-1 == cfsetospeed(&(handle->settings), B115200)) {
                return -1;
        }

        /*
                Enable the reciever and ignore control lines.

                I would enable hardware flow control (CRTSCTS) but it is not
                defined in the ansi standard.
        */
        handle->settings.c_cflag |= (CLOCAL | CREAD);

        /*
                Set up raw input mode and raw output mode.
        */
        handle->settings.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);
        handle->settings.c_oflag &= ~OPOST;

        /*
                Set the character transmission to 8 bits, 1 stop, no parity.
        */
        handle->settings.c_cflag &= ~PARENB;
        handle->settings.c_cflag &= ~CSTOPB;
        handle->settings.c_cflag &= ~CSIZE;
        handle->settings.c_cflag |= CS8;

        /*
                We don't expect any bytes, and we allow a 10 * 100 ms
                (1 second) delay.
        */
        handle->settings.c_cc[VMIN] = 0;
        handle->settings.c_cc[VTIME] = 10;

        /*
                Try to apply the new settings, and apply them immediately.
        */
        if (-1 == tcsetattr(handle->fd, TCSANOW, &(handle->settings))) {
                return -1;
        }

        /*
                Flush any data in the buffer.
        */
        tcflush(handle->fd, TCIOFLUSH);

        /*
                Send our initialization string.
        */
        if (-1 == modem_write(handle, init_string, strlen(init_string))) {
                return -1;
        }

        /*
                Wait a little bit for the modem to respond to the
                initialization string. We will try this MODEM_READ_RETRIES
                times (default is 5).
        */
        len = 0;
        retries = 0;
        do {
                /* Sleep for a bit to let it initialize. */
                paragon_usleep(PARAGON_ACM_INIT_WAIT);
                response = modem_read(handle, &len);
                retries++;
        } while (retries < MODEM_READ_RETRIES && response == NULL);

        parsed = trim(response);
        init_ok = strncmp(getlastline(parsed), expected, strlen(expected));

        /*
                Release our temporaries.
        */
        free((void *)response);
        free((void *)parsed);

        /*
                Flush any old data.
        */
        tcflush(handle->fd, TCIOFLUSH);

        /*
                We didn't get the expected response, so we assume there was
                a problem. There really should be a better way of doing this.
        */
        if (0 != init_ok) {
                return -1;
        }

        return 0;
}

int modem_cleanup(modem_t *handle)
{
        if (NULL == handle) {
                return -1;
        }

        /* Don't bother dealing with invalid arguments. */
        if (-1 == handle->fd) {
                return 0;
        }

        /* Flush any pending requests. */
        tcflush(handle->fd, TCIOFLUSH);

        /* Let's return the modem to the state it was in when we found it. */
        if (tcsetattr(handle->fd, TCSANOW, &(handle->old_settings)) == -1) {
                return -1;
        }

        return 0;
}

char *modem_read(modem_t *handle, int *bytes)
{
        char *s;
        int result;

        if (NULL == handle) {
                return NULL;
        }

        /*
                Result will be -1 on error, but mainly the ioctl() call tells
                us if the file descriptor (our modem) has any data to be read.
                In that case, bytes will be greater than 0.
        */
        result = ioctl(handle->fd, FIONREAD, bytes);
        if (result != -1 && *bytes > 0) {
                s = (char *)malloc(*bytes + 1);
                read(handle->fd, s, *bytes);
                s[*bytes] = '\0';
                return s;
        }

        return NULL;
}

int modem_write(modem_t *handle, char *data, int bytes)
{
        int wrote;

        if (NULL == handle) {
                return -1;
        }

        if (NULL == data || bytes <= 0) {
                return bytes;
        }

        /* Send our data to the modem. */
        wrote = write(handle->fd, data, bytes);

        /* For some reason not all bytes were written. This is an error. */
        if (wrote < bytes) {
                return -1;
        }

        return bytes;
}
/* ------------------------------------------------------------------------ */
