/**
        \file modes.h
        \author Sachith Dunatunga
        \date 21.04.09
        \brief The header/interface for the modeswitching functions.
*/
#ifndef __PARAGON_MODES_H__
#define __PARAGON_MODES_H__
#include "handle.h"

/* ---libparagon----------------------------------------------------------- */
/**
        \defgroup modes Phone Modes
                The group of modes the phone may switch between.
        @{
*/
/**
        \brief Used during initialization to force a guess of phone mode.
*/
#define PARAGON_MODE_AUTO (0)
/**
        \brief The phone is in AT mode.

        In this mode, the phone uses a modem interface but is not claimed by
        the normal Linux kernel driver (it does not have an associated
        tty device).
*/
#define PARAGON_MODE_AT (1)
/**
        \brief The phone is in ACM (Modem) mode.

        This mode is really the same as AT mode except the interface has been
        claimed by a kernel driver. Thus, we must use either detach the
        kernel driver or go through the tty device to communicate with the
        phone.
*/
#define PARAGON_MODE_ACM (2)
/**
        \brief The phone is in Memcard mode.

        In this mode, the phone presents itself as a mass storage device. It
        may or may not be claimed by a kernel driver, but it must be detached
        from the kernel if we are to issue commands to it.
*/
#define PARAGON_MODE_MEMCARD (3)
/**
        \brief The phone is in P2K/P2K05 mode.

        This is the mode we usually want when we're using libparagon. It
        is the mode that allows us to read the phone filesystem and seems,
        which are basically motorola data records.
*/
#define PARAGON_MODE_P2K (4)
/**
        \brief The phone is in an unstable state.

        We don't know what mode the phone is in. This sometimes occurs
        if there is an error in the middle of a modeswitch; in that case, we
        don't know what mode the phone is in now. If we try to reconnect to
        the phone, we treat this as PARAGON_MODE_AUTO.
*/
#define PARAGON_MODE_UNKNOWN (5)
/**
        @}
*/

/**
        \brief Switches the phone to the requested mode.
        \param handle The libparagon handle to use.
        \param mode The mode to switch to.
        \return PARAGON_SUCCESS on success, an integer less than 0 on error.

        This function just wraps around the more specific mode switching
        functions. It will return PARAGON_BADARG if the mode is invalid.
*/
int paragon_set_mode(paragon_handle_t *handle, int mode);
/**
        \brief Gets the mode of the phone.
        \param handle The libparagon handle to use.
        \param mode A pointer to store the phone mode.
        \return PARAGON_SUCCESS on success, an integer less than 0 on error.
*/
int paragon_get_mode(paragon_handle_t *handle, int *mode);
/* ------------------------------------------------------------------------ */
#endif
