/**
        \file paragon.h
        \author Sachith Dunatunga
        \date 21.04.09
        \brief The header/interface for libparagon.
*/
#ifndef __PARAGON_H__
#define __PARAGON_H__
#include "command.h"
#include "error.h"
#include "fsac.h"
#include "handle.h"
#include "logger.h"
#include "modes.h"
#include "seem.h"

/* ---libparagon----------------------------------------------------------- */
/**
        \mainpage libparagon - A library for connecting to P2K/P2K05 phones

        \section Overview
                In the past, Motorola has made an enormous number of
        cell phones, including the ever popular Motorola RAZR. Unfortunately,
        drivers for communication with the phone exist only for the Windows
        platform in the form of its software suite Motorola Phone Tools.
        Complicating matters further is the introduction of new phones with
        a different external interface compared to the older models, although
        superficially they appear identical (using the Paragon 2000 and
        Paragon 2005 interfaces). While Motorola's popularity has waned in
        recent years, there are still many motorola phones available on the
        market. This library attempts to bring most of the functionality of
        the Motorola Phone Tools to the Linux platform, and is designed to
        work with both Paragon 2000 and Paragon 2005 phones in a manner
        transparent to the application developer.

        \section Terminology
                The motorola modding community is quite vibrant and has come
        up with some colorful terms for motorola features. Here are a few of
        the core terms used:
        - \b P2K
                This stands for Paragon 2000, which was found in Motorola's
                own software suite for connecting to phones; it is the name of
                their file system on older phones such as the original RAZR
                V3. Although it has been superceded by P2K05 and USBLAN, most
                commands are still referred to as 'P2K' commands although the
                underlying protocols may in fact be different.
        - \b P2K05
                This stands for Paragon 2005, a newer file system found on
                most older 3G motorola phones such as the RAZR V3xx and
                RAZR V6 Maxx. It supports all the P2K commands in addition to
                a few others; however, to remain portable, libparagon does not
                make use of any P2K05 specific commands.
        - \b USBLAN
                This is motorola's newest filesystem found on modern high end
                phones; I don't have any experience with these phones, but
                from what I gather, it mostly uses an FTP protocol to do file
                transfers. Libparagon currently \b does \b not support this
                filesystem.
        - \b Seem
                I don't know the history behind this term, but the motorola
                modding community has settled on 'seem' to describe an
                element in a binary database in the phone. It is not
                accessible through the file system, so perhaps it is stored in
                NVRAM. Perhaps the most comparable analogue is the system BIOS
                in computers; it stores certain settings that cannot usually
                be accessed through a file system. Motorola's software suite
                actually refers to these as 'elements', but every single
                modding program written thus far has stuck with the term
                'seem'.
        .

        \section Features
                Libparagon is quite a simple library, although you can
        easily extend it to include other commands. However, it does support
        a few core features:
        - Protocols supported
                - Paragon 2000 (P2K)
                - Paragon 2005 (P2K05)
        - Seem Access
                - Read Seems
                - Write Seems
                .
        - Volume Access
                - Get Free Space
                - Get Number of Files
                .
        - Directory Access
                - Create Directories
                - Remove Directories
                .
        - File Access
                - Create Files
                - Write Files
                - Read Files
                - Delete Files
                - Seek within Files
                - Change File Attributes
                .
        .

        \section Usage
        \subsection core Core Commands
                Most libparagon programs will call the following functions
        (roughly in order):
        -# paragon_init()
                Used to setup the libparagon handle. Every program that uses
                libparagon will use this.
        -# paragon_get_mode()
                Used to get the current mode of the phone; we use this to
                know what state to revert to upon deinitialization.
        -# paragon_set_mode()
                Used to force the phone into P2K mode; most of the time
                the phone is not in the correct mode when connected.
        -# paragon_fini()
                Used to close the libparagon handle and revert all changes
                possible. Note that we won't reattach kernel drivers if they
                were detached.
        .
        \subsection data_transfers Data Transfer Commands
                Optionally, usually some sort of data transfers will take
        place; for this we may use the following functions:
        -# paragon_set_file_mask()
                We call this function to set the phones internal file list; it
                is okay to call this function with a wildcard (*) to select
                more than one file at a time.
        -# paragon_open_file()
                Now we can actually open the file on the phone; we merely
                supply the path and the phone takes care of the rest. The file
                is created if it does not already exist. Note that attributes
                are changed upon opening, so use the
                paragon_get_next_file_record() command to get the original
                attributes of the file to save them; alternatively, calling
                this function with bogus attributes (such as 0xFFFF) will
                result in keeping the original attributes, although I don't
                know if/when this behaviour will break. Upon opening, the
                file position marker is right at the beginning (byte 0).
        -# paragon_read_file()
                This is probably the heart of most programs (along with
                paragon_write_file()). We can read an abitrary number of bytes
                from the current position in a file; in conjuction with the
                paragon_get_next_file_record() function, we can read the
                entire file into a buffer, which can then be used by the
                calling application for any purpose.
        -# paragon_seek_file()
                This function is extremely poorly documented; however,
                calling it with both offset and direction set to 0 will seek
                to the beginning of the file.
        -# paragon_write_file()
                The write file command is used to write an arbitrary number of
                bytes to the current position in a file; the file is
                automatically made larger if we write past the EOF.
        -# paragon_close_file()
                We use this command to close the file on the phone; keeping a
                file open will stop other processes on the phone itself from
                accessing it, so it is important to close the file. A soft
                reset will close all open file handles.
        -# paragon_read_seem()
                We use this command to read a seem into a buffer; we can
                specifiy what length, what offset, and which seem we want.
        -# paragon_write_seem()
                We use this command to write a buffer as a seem to the phone.
                Again we can specifiy what length, what offset, and which
                seem to write.
        .

        \subsection outline Outline
        The general outline for a libparagon program is as follows:
        \code
        paragon_init()
        paragon_get_mode()
        paragon_set_mode()
        [...data transfers follow...]
        paragon_read_seem()
        paragon_write_seem()
        paragon_set_file_mask()
        paragon_open_file()
        paragon_read_file()
        paragon_seek_file()
        paragon_write_file()
        paragon_close_file()
        [...end data transfers...]
        paragon_fini()
        \endcode

        See the example file \link p2k_example.c p2k_example.c \endlink for
        a fully working program using libparagon.
*/

/**
        \brief This function initializes libparagon.
        \param handle The libparagon handle to save. This should be a valid
        pointer and if the call is successful, libparagon data will be saved
        to it.
        \param params The parameters to initialize libparagon with. Passing
        in NULL results in a default set of parameters being used.
        \return PARAGON_SUCCESS on success, an integer less than 0 on failure.

        This function takes care of all the low level details like
        scanning the usb bus for the phone and setting up the libparagon
        handle structure for use. The default set of parameters implied by
        passing NULL through 'params' consists of any vendor ID,
        any product ID, automatic mode selection, and no ACM device.
*/
int paragon_init(paragon_handle_t *handle, paragon_init_t *params);

/**
        \brief This function cleans up libparagon.
        \param handle The libparagon handle to cleanup. It should have first
        been initialized with paragon_init().
        \return PARAGON_SUCCESS on success, an integer less than 0 on failure.

        This function tries to undo everything paragon_init() did, at least
        where possible. One thing it will not do is reattach any kernel
        drivers to the phone, and it will not return the phone to the
        original mode in some cases.
*/
int paragon_fini(paragon_handle_t *handle);
/**
        \example p2k_example.c

        This example uses libparagon to retrieve the phone name from the phone
        by reading a seem (a form of motorola data record).
*/
/* ------------------------------------------------------------------------ */
#endif
